module.exports = (function () {
    var colors = require('colors').setTheme({
        filename: ['white', 'bold'],
        warning: ['yellow', 'bold'],
        error: ['red', 'bold']
    });
    var getChar = require('cli-interact').getChar;

    function question(message) {
        var answer = getChar(message, 'yn', true);
        switch (answer) {
            case 'y':
            case 'Y':
                break;
            case 'n':
            case 'N':
            default:
                process.exit(0);
        }
    }

    var _level = 1;
    var _code = [];

    function _Error(message, filename, loc) {
        this.message = message;
        this.filename = filename;
        this.lineNumber = loc.start.line;
        this.columnNumber = loc.start.column;
        ReferenceError.call(this, message, filename, loc.start.line);
    }
    _Error.prototype = Object.create(ReferenceError.prototype);
    _Error.prototype.constructor = _Error;

    function warning(message, filename, loc) {
        var codeFragment = getCodeFragment(_code, loc);

        message = [
            [loc.start.line, loc.start.column].join(':'),
            codeFragment,
            message.warning
        ].join('\n\n');

        filename = '['+ filename.filename +']';

        switch (_level) {
            case 0:
                throw new _Error(message, filename, loc);
            default:
                question('ReferenceWarning: '+ filename+ ' ' +message + ' Skip?');
        }
    }

    function error(message, filename, loc) {
        var codeFragment = getCodeFragment(_code, loc);

        message = [
            [loc.start.line, loc.start.column].join(':'),
            codeFragment,
            message.error
        ].join('\n\n');

        filename = '['+ filename.filename +']';
        
        switch (_level) {
            case 0:
            case 1:
                throw new _Error(message, filename, loc);
            default:
                question('ReferenceError: '+ filename+ ' ' +message + ' Skip?');        
        }
    }

    function setLevel(level) {
        if (typeof(level) === 'number') {
            _level = level;
        }
    }

    function setSourceCode(code) {
        if (typeof(code) === 'string') {
            _code = code.split(/\n/);
        }   
    }

    function getCodeFragment(code, loc) {
        var start;
        var markLineIdx = 1;
        start = (start = loc.start.line - 2) < 0 ? 0 : start;

        return code.slice(start, loc.end.line + 1).map(function (line, idx, arr) {
            var outputLine;
            if (arr.length === 2 && start === 0) {
                markLineIdx = 0;
            }
            if (idx === markLineIdx) {
                outputLine = line.slice(0, loc.start.column).green + line.slice(loc.start.column, loc.end.column).underline.red + line.slice(loc.end.column).green;
            } else {
                outputLine = line.green;
            }
            return (loc.start.line + idx - 1 + ' ').gray + outputLine;
        }).join('\n');
    }

    return {
        warning: warning,
        error: error,
        setLevel: setLevel,
        setSourceCode: setSourceCode
    }
})();